import TimedActionManager from './timed-actions';

const SAVE_DELAY = 3000;

export default class AutoSaveManager {
  private static timedActionsManager = TimedActionManager.instance;
  
  /**
   * Enqueues an update on "element" performed by "action" in at most "delay" milliseconds.
   * @param id a key that allows us to identify the element in the save queues
   * @param element the element that needs to be updated
   * @param action the save method, should accept a key and an element as parameters.
   * @param delay (optional) a custom delay time to perform the update. If ommited, SAVE_DELAY is used
   */
  public static save(id : string, element : object, action : (key: string, element: object) => Promise<any>, delay? : number) : Promise<any>{
    return AutoSaveManager.timedActionsManager.start(id, element, (key : string, el : object) => {
        return action(key, el);
      }, delay || SAVE_DELAY);
  } 
    
  /**
   * Cancels the future execution of a save action on an element identified by "id"
   * @param id the key of the queued actions for an element
   */
  public static cancel(id : string) {
      return AutoSaveManager.timedActionsManager.cancel(id);
  }

  /**
   * Executes action on element immediately. 
   * @param id a key that allows us to identify the element in the save queues
   * @param element the element that needs to be updated
   * @param action the save method, should accept a key and an element as parameters.
   */
  public static saveNow(id : string, element : object, action : (key: string, element: object) => Promise<any>) : Promise<any>{
    return AutoSaveManager.timedActionsManager.immediate(id, element, (key : string, el : object) => {
        return action(key, el);
    }); 
  }
}