import { expect } from 'chai';
import { MemCache } from '../../cache';

describe('Cache storage', () => {
  it('should store items by key', (done) => {
    const data = {
      id: 'id',
      content: 'some random content',
    };
    MemCache.add('key', data);
    expect(MemCache.retrieve('key').id).to.be.equal(data.id);
    done();
  });

});

describe('Cache validation', () => {
  it('should preserve keys as valid if accessed immediately', (done) => {
    MemCache.add('key', 'item');
    MemCache.add('key3', 'another item');
    expect(MemCache.isValid('key')).to.be.true;
    expect(MemCache.isValid('key3')).to.be.true;
    done();
  });
});

describe('Cache invalidation via timeout', () => {
  it('should automatically invalidate items', (done) => {
    MemCache.add('key', 'something', 100);
    setTimeout(() => {
      expect(MemCache.retrieve('key')).to.be.equal(null);
      done();
    }, 200);
  });
});

